@@ -55,6 +55,8 @@ pub struct Compiler<'a> {
     scope_depth: i32,
     current_card: i32,
     current_lane: Box<str>,
+    /// currently processed module's cards
+    current_cards: Option<&'a ModuleCards>,
 }
 
 #[derive(Debug, Clone, Copy)]
@@ -103,6 +105,7 @@ impl<'a> Compiler<'a> {
             current_card: -1,
             current_lane: "".into(),
             current_imports: Default::default(),
+            current_cards: None,
         }
     }
 
@@ -185,19 +188,11 @@ impl<'a> Compiler<'a> {
         let mut lanes = compilation_unit.iter().enumerate();
 
         if let Some((il, main_lane)) = lanes.next() {
-            let len: u32 = match main_lane.cards.len().try_into() {
-                Ok(i) => i,
-                Err(_) => return Err(self.error(CompilationErrorPayload::TooManyCards(il))),
-            };
             self.scope_begin();
             self.process_lane(il, main_lane, 0)?;
-            let nodeid = NodeId {
-                lane: il as u32,
-                pos: len,
-            };
             self.scope_end();
             // insert explicit exit after the first lane
-            self.process_card(nodeid, &Card::Abort)?;
+            self.push_instruction(Card::Abort.instruction().unwrap());
         }
 
         for (il, lane) in lanes {
@@ -270,9 +265,11 @@ impl<'a> Compiler<'a> {
             cards,
             namespace,
             imports,
+            card_impls,
         }: &'a LaneIr,
         instruction_offset: i32,
     ) -> CompilationResult<()> {
+        self.current_cards = Some(card_impls);
         self.current_lane = name.clone();
         self.current_card = -1;
         self.current_namespace = Cow::Borrowed(namespace);
@@ -287,13 +284,13 @@ impl<'a> Compiler<'a> {
         for param in arguments.iter() {
             self.add_local(param.as_str())?;
         }
-        for (ic, card) in cards.iter().enumerate() {
+        for (ic, card_id) in cards.iter().enumerate() {
             self.current_card = ic as i32;
             let nodeid = NodeId {
                 lane: il as u32,
                 pos: (ic as i32 + instruction_offset) as u32,
             };
-            self.process_card(nodeid, card)?;
+            self.process_card(nodeid, *card_id)?;
         }
         Ok(())
     }
@@ -327,8 +324,20 @@ impl<'a> Compiler<'a> {
     }
 
     fn encode_jump(&mut self, lane: &LaneNode) -> CompilationResult<()> {
+        let to = self.lookup_lane(&self.jump_table, lane)?;
+        write_to_vec(to.hash_key, &mut self.program.bytecode);
+        write_to_vec(to.arity, &mut self.program.bytecode);
+        Ok(())
+    }
+
+    // take jump_table by param because of lifetimes
+    fn lookup_lane<'b>(
+        &self,
+        jump_table: &'b KeyMap<LaneMeta>,
+        lane: &LaneNode,
+    ) -> CompilationResult<&'b LaneMeta> {
         // attempt to look up the function by name
-        let mut to = self.jump_table.get(Handle::from(lane));
+        let mut to = jump_table.get(Handle::from(lane));
         if to.is_none() {
             // attempt to look up the function in the current namespace
 
@@ -339,7 +348,7 @@ impl<'a> Compiler<'a> {
                     .flat_map(|x| [x.as_bytes(), ".".as_bytes()])
                     .chain(std::iter::once(lane.0.as_bytes())),
             );
-            to = self.jump_table.get(handle);
+            to = jump_table.get(handle);
         }
         if to.is_none() {
             // attempt to look up function in the imports
@@ -348,24 +357,52 @@ impl<'a> Compiler<'a> {
                 .iter()
                 .find(|(import, _)| *import == &lane.0)
             {
+                let (super_depth, suffix) = super_depth(alias);
                 let handle = Handle::from_bytes_iter(
                     self.current_namespace
                         .iter()
+                        .take(self.current_namespace.len() - super_depth)
                         .flat_map(|x| [x.as_bytes(), ".".as_bytes()])
-                        .chain(std::iter::once(alias.as_bytes())),
+                        .chain(std::iter::once(suffix.unwrap_or(alias).as_bytes())),
                 );
-                to = self.jump_table.get(handle);
+                to = jump_table.get(handle);
             }
         }
-        let to = to.ok_or_else(|| {
+        if to.is_none() {
+            // attempt to look up by imported module
+            if let Some((prefix, suffix)) = lane.0.as_str().split_once('.') {
+                if let Some((_, alias)) = self
+                    .current_imports
+                    .iter()
+                    .find(|(import, _)| *import == prefix)
+                {
+                    // namespace.alias.suffix
+                    let (super_depth, s) = super_depth(alias);
+                    let handle = Handle::from_bytes_iter(
+                        self.current_namespace
+                            .iter()
+                            .take(self.current_namespace.len() - super_depth)
+                            .flat_map(|x| [x.as_bytes(), ".".as_bytes()])
+                            .chain(
+                                [
+                                    alias.as_bytes(),
+                                    ".".as_bytes(),
+                                    s.unwrap_or(suffix).as_bytes(),
+                                ]
+                                .iter()
+                                .copied(),
+                            ),
+                    );
+                    to = jump_table.get(handle);
+                }
+            }
+        }
+        to.ok_or_else(|| {
             self.error(CompilationErrorPayload::InvalidJump {
                 dst: lane.clone(),
                 msg: None,
             })
-        })?;
-        write_to_vec(to.hash_key, &mut self.program.bytecode);
-        write_to_vec(to.arity, &mut self.program.bytecode);
-        Ok(())
+        })
     }
 
     fn push_str(&mut self, data: &str) {
@@ -375,17 +412,17 @@ impl<'a> Compiler<'a> {
         encode_str(data, &mut self.program.data);
     }
 
-    fn resolve_var(&self, name: &str) -> CompilationResult<isize> {
+    fn resolve_var(&self, name: &str) -> CompilationResult<Option<usize>> {
         self.validate_var_name(name)?;
         for (i, local) in self.locals.iter().enumerate().rev() {
             if local.name == name {
-                return Ok(i as isize);
+                return Ok(Some(i));
             }
         }
-        Ok(-1)
+        Ok(None)
     }
 
-    fn process_card(&mut self, nodeid: NodeId, card: &'a Card) -> CompilationResult<()> {
+    fn process_card(&mut self, nodeid: NodeId, card: CardId) -> CompilationResult<()> {
         let handle = u32::try_from(self.program.bytecode.len())
             .expect("bytecode length to fit into 32 bits");
         let nodeid_hash = Handle::from_u64(nodeid.into());
@@ -395,6 +432,11 @@ impl<'a> Compiler<'a> {
             .insert(nodeid_hash, Label::new(handle))
             .unwrap();
 
+        let card = self
+            .current_cards
+            .expect("Cards not set")
+            .get(&card)
+            .ok_or_else(|| self.error(CompilationErrorPayload::MissingCard { card_id: card }))?;
         if let Some(instr) = card.instruction() {
             // instruction itself
             self.push_instruction(instr);
@@ -402,8 +444,8 @@ impl<'a> Compiler<'a> {
         match card {
             Card::Noop => {}
             Card::CompositeCard { cards, .. } => {
-                for card in cards.iter() {
-                    self.process_card(nodeid, card)?;
+                for card_id in cards.iter() {
+                    self.process_card(nodeid, *card_id)?;
                 }
             }
             Card::ForEach { variable, lane } => {
@@ -499,24 +541,24 @@ impl<'a> Compiler<'a> {
             } => {
                 let mut idx = 0;
                 self.encode_if_then(Instruction::GotoIfFalse, |c| {
-                    c.process_card(nodeid, then_card)?;
+                    c.process_card(nodeid, *then_card)?;
                     // jump over the `else` branch
                     c.push_instruction(Instruction::Goto);
                     idx = c.program.bytecode.len();
                     write_to_vec(0i32, &mut c.program.bytecode);
                     Ok(())
                 })?;
-                self.process_card(nodeid, else_card)?;
+                self.process_card(nodeid, *else_card)?;
                 unsafe {
                     let ptr = self.program.bytecode.as_mut_ptr().add(idx) as *mut i32;
                     std::ptr::write_unaligned(ptr, self.program.bytecode.len() as i32);
                 }
             }
             Card::IfFalse(jmp) => {
-                self.encode_if_then(Instruction::GotoIfTrue, |c| c.process_card(nodeid, jmp))?
+                self.encode_if_then(Instruction::GotoIfTrue, |c| c.process_card(nodeid, *jmp))?
             }
             Card::IfTrue(jmp) => {
-                self.encode_if_then(Instruction::GotoIfFalse, |c| c.process_card(nodeid, jmp))?
+                self.encode_if_then(Instruction::GotoIfFalse, |c| c.process_card(nodeid, *jmp))?
             }
             Card::Jump(jmp) => self.encode_jump(jmp)?,
             Card::StringLiteral(c) => self.push_str(c.0.as_str()),
@@ -560,31 +602,34 @@ impl<'a> Compiler<'a> {
 
     fn read_var_card(&mut self, variable: &VarNode) -> CompilationResult<()> {
         let scope = self.resolve_var(variable.0.as_str())?;
-        if scope < 0 {
-            // global
-            let next_var = &mut self.next_var;
-            let varhash = Handle::from_bytes(variable.0.as_bytes());
-            let id = self
-                .program
-                .variables
-                .ids
-                .entry(varhash)
-                .or_insert_with(move || {
-                    let id = *next_var;
-                    *next_var = VariableId(id.0 + 1);
-                    id
-                });
-            let id = *id;
-            self.program
-                .variables
-                .names
-                .entry(Handle::from_u32(id.0))
-                .or_insert_with(|| *variable.0);
-            self.push_instruction(Instruction::ReadGlobalVar);
-            write_to_vec(id, &mut self.program.bytecode);
-        } else {
-            //local
-            self.read_local_var(scope as u32);
+        match scope {
+            Some(scope) => {
+                //local
+                self.read_local_var(scope as u32);
+            }
+            None => {
+                // global
+                let next_var = &mut self.next_var;
+                let varhash = Handle::from_bytes(variable.0.as_bytes());
+                let id = self
+                    .program
+                    .variables
+                    .ids
+                    .entry(varhash)
+                    .or_insert_with(move || {
+                        let id = *next_var;
+                        *next_var = VariableId(id.0 + 1);
+                        id
+                    });
+                let id = *id;
+                self.program
+                    .variables
+                    .names
+                    .entry(Handle::from_u32(id.0))
+                    .or_insert_with(|| *variable.0);
+                self.push_instruction(Instruction::ReadGlobalVar);
+                write_to_vec(id, &mut self.program.bytecode);
+            }
         }
         Ok(())
     }
@@ -612,3 +657,16 @@ impl<'a> Compiler<'a> {
         self.program.bytecode.push(instruction as u8);
     }
 }
+
+fn super_depth(import: &str) -> (usize, Option<&str>) {
+    let mut super_pog = import.split_once("super.");
+    let mut super_cnt = 0;
+    let mut suffix = None;
+    while let Some((_sup_pre, sup_post)) = super_pog {
+        super_cnt += 1;
+        super_pog = sup_post.split_once("super.");
+        suffix = Some(sup_post);
+    }
+
+    (super_cnt, suffix)
+}
